@@ -10,24 +10,22 @@ from orient.hints import (Contour,
                           Region,
                           Segment)
 from . import bounding_box
-from .contour import (_relate_segment as relate_segment_to_contour,
-                      edges as boundary_edges,
-                      equal as contours_equal,
-                      register as register_contour)
-from .events_queue import EventsQueue
+from .contour import (equal as contours_equal,
+                      point_vertex_line_divides_angle,
+                      to_segments as contour_to_segments)
 from .processing import (process_compound_queue,
                          process_linear_compound_queue)
 from .relation import Relation
-from .segment import relate_point as relate_point_to_segment
+from .segment import (relate_point as relate_point_to_segment,
+                      relate_segment as relate_segments)
+from .sweep import ClosedSweeper
+from .utils import flatten
 
 
 def relate_point(region: Region, point: Point) -> Relation:
-    if not bounding_box.contains_point(bounding_box.from_points(region),
-                                       point):
-        return Relation.DISJOINT
     result = False
     _, point_y = point
-    for edge in boundary_edges(region):
+    for edge in to_segments(region):
         if relate_point_to_segment(edge, point) is Relation.COMPONENT:
             return Relation.COMPONENT
         start, end = edge
@@ -41,11 +39,59 @@ def relate_point(region: Region, point: Point) -> Relation:
             else Relation.DISJOINT)
 
 
+def _relate_segment_to_contour(contour: Contour, segment: Segment) -> Relation:
+    # similar to segment-in-contour check
+    # but cross has higher priority over overlap
+    # because cross with contour will be considered as cross with region
+    # whereas overlap with contour can't be an overlap with region
+    # and should be classified by further analysis
+    has_no_touch = has_no_overlap = True
+    last_touched_edge_index = last_touched_edge_start = None
+    start, end = segment
+    for index, edge in enumerate(to_segments(contour)):
+        relation_with_edge = relate_segments(edge, segment)
+        if (relation_with_edge is Relation.COMPONENT
+                or relation_with_edge is Relation.EQUAL):
+            return Relation.COMPONENT
+        elif (relation_with_edge is Relation.OVERLAP
+              or relation_with_edge is Relation.COMPOSITE):
+            if has_no_overlap:
+                has_no_overlap = False
+        elif relation_with_edge is Relation.TOUCH:
+            edge_start, edge_end = edge
+            if has_no_touch:
+                has_no_touch = False
+            elif (index - last_touched_edge_index == 1
+                  and start not in edge and end not in edge
+                  and (angle_orientation(end, start, edge_start)
+                       is Orientation.COLLINEAR)
+                  and point_vertex_line_divides_angle(start,
+                                                      last_touched_edge_start,
+                                                      edge_start, edge_end)):
+                return Relation.CROSS
+            last_touched_edge_index = index
+            last_touched_edge_start = edge_start
+        elif relation_with_edge is Relation.CROSS:
+            return Relation.CROSS
+    if not has_no_touch and last_touched_edge_index == len(contour) - 1:
+        first_edge = first_edge_start, first_edge_end = contour[-1], contour[0]
+        if (relate_segments(first_edge, segment) is Relation.TOUCH
+                and start not in first_edge and end not in first_edge
+                and (angle_orientation(end, start, first_edge_start)
+                     is Orientation.COLLINEAR)
+                and point_vertex_line_divides_angle(start, contour[-2],
+                                                    first_edge_start,
+                                                    first_edge_end)):
+            return Relation.CROSS
+    return ((Relation.DISJOINT
+             if has_no_touch
+             else Relation.TOUCH)
+            if has_no_overlap
+            else Relation.OVERLAP)
+
+
 def relate_segment(region: Region, segment: Segment) -> Relation:
-    if bounding_box.disjoint_with(bounding_box.from_points(region),
-                                  bounding_box.from_points(segment)):
-        return Relation.DISJOINT
-    relation_with_contour = relate_segment_to_contour(region, segment)
+    relation_with_contour = _relate_segment_to_contour(region, segment)
     if (relation_with_contour is Relation.CROSS
             or relation_with_contour is Relation.COMPONENT):
         return relation_with_contour
@@ -69,7 +115,7 @@ def relate_segment(region: Region, segment: Segment) -> Relation:
             start_index = end_index = None
             start_is_not_vertex = end_is_not_vertex = True
             overlaps_with_edges = relation_with_contour is Relation.OVERLAP
-            for index, edge in enumerate(boundary_edges(region)):
+            for index, edge in enumerate(to_segments(region)):
                 if (start_index is None
                         and (relate_point_to_segment(edge, start)
                              is Relation.COMPONENT)):
@@ -215,6 +261,26 @@ def relate_segment(region: Region, segment: Segment) -> Relation:
                     else Relation.TOUCH)
 
 
+def relate_multisegment(region: Region, multisegment: Contour) -> Relation:
+    if not multisegment:
+        return Relation.DISJOINT
+    multisegment_bounding_box, region_bounding_box = (
+        bounding_box.from_points(flatten(multisegment)),
+        bounding_box.from_points(region))
+    if bounding_box.disjoint_with(multisegment_bounding_box,
+                                  region_bounding_box):
+        return Relation.DISJOINT
+    sweeper = ClosedSweeper()
+    sweeper.register_segments(to_segments(region),
+                              from_test=False)
+    sweeper.register_segments(multisegment,
+                              from_test=True)
+    (_, multisegment_max_x, _, _), (_, region_max_x, _, _) = (
+        multisegment_bounding_box, region_bounding_box)
+    return process_linear_compound_queue(sweeper, min(multisegment_max_x,
+                                                      region_max_x))
+
+
 def relate_contour(region: Region, contour: Contour) -> Relation:
     contour_bounding_box, region_bounding_box = (
         bounding_box.from_points(contour), bounding_box.from_points(region))
@@ -222,15 +288,15 @@ def relate_contour(region: Region, contour: Contour) -> Relation:
         return Relation.DISJOINT
     if equal(region, contour):
         return Relation.COMPONENT
-    events_queue = EventsQueue()
-    register(events_queue, region,
-             from_test=False)
-    register_contour(events_queue, contour,
-                     from_test=True)
+    sweeper = ClosedSweeper()
+    sweeper.register_segments(to_segments(region),
+                              from_test=False)
+    sweeper.register_segments(contour_to_segments(contour),
+                              from_test=True)
     (_, contour_max_x, _, _), (_, region_max_x, _, _) = (contour_bounding_box,
                                                          region_bounding_box)
-    return process_linear_compound_queue(events_queue, min(contour_max_x,
-                                                           region_max_x))
+    return process_linear_compound_queue(sweeper, min(contour_max_x,
+                                                      region_max_x))
 
 
 def relate_region(goal: Region, test: Region) -> Relation:
@@ -240,15 +306,15 @@ def relate_region(goal: Region, test: Region) -> Relation:
         return Relation.DISJOINT
     if equal(goal, test):
         return Relation.EQUAL
-    events_queue = EventsQueue()
-    register(events_queue, goal,
-             from_test=False)
-    register(events_queue, test,
-             from_test=True)
+    sweeper = ClosedSweeper()
+    sweeper.register_segments(to_segments(goal),
+                              from_test=False)
+    sweeper.register_segments(to_segments(test),
+                              from_test=True)
     (_, goal_max_x, _, _), (_, test_max_x, _, _) = (goal_bounding_box,
                                                     test_bounding_box)
-    return process_compound_queue(events_queue, min(goal_max_x, test_max_x))
+    return process_compound_queue(sweeper, min(goal_max_x, test_max_x))
 
 
 equal = contours_equal
-register = register_contour
+to_segments = contour_to_segments
