@@ -5,6 +5,8 @@ from typing import (List,
 Coordinate = Real
 Point = Tuple[Coordinate, Coordinate]
 Segment = Tuple[Point, Point]
-Contour = List[Point]
+Multisegment = List[Segment]
+Contour = Region = List[Point]
 Multicontour = List[Contour]
-Polygon = Tuple[Contour, Multicontour]
+Multiregion = List[Region]
+Polygon = Tuple[Region, Multiregion]
