@@ -1,11 +1,13 @@
 from .core import (contour as _contour,
                    multiregion as _multiregion,
+                   multisegment as _multisegment,
                    polygon as _polygon,
                    region as _region,
                    segment as _segment)
 from .core.relation import Relation
 from .hints import (Contour,
                     Multiregion,
+                    Multisegment,
                     Point,
                     Polygon,
                     Region,
@@ -74,6 +76,116 @@ def segment_in_segment(left: Segment, right: Segment) -> Relation:
     return _segment.relate_segment(right, left)
 
 
+def point_in_multisegment(point: Point,
+                          multisegment: Multisegment) -> Relation:
+    """
+    Finds relation between point and multisegment.
+
+    Time complexity:
+        ``O(len(multisegment))``
+    Memory complexity:
+        ``O(1)``
+
+    :param point: point to check for.
+    :param multisegment: multisegment to check in.
+    :returns: relation between point and multisegment.
+
+    >>> multisegment = [((0, 0), (1, 0)), ((3, 0), (5, 0))]
+    >>> point_in_multisegment((0, 0), multisegment) is Relation.COMPONENT
+    True
+    >>> point_in_multisegment((0, 1), multisegment) is Relation.DISJOINT
+    True
+    >>> point_in_multisegment((1, 0), multisegment) is Relation.COMPONENT
+    True
+    >>> point_in_multisegment((2, 0), multisegment) is Relation.DISJOINT
+    True
+    >>> point_in_multisegment((3, 0), multisegment) is Relation.COMPONENT
+    True
+    >>> point_in_multisegment((4, 0), multisegment) is Relation.COMPONENT
+    True
+    """
+    return _multisegment.relate_point(multisegment, point)
+
+
+def segment_in_multisegment(segment: Segment,
+                            multisegment: Multisegment) -> Relation:
+    """
+    Finds relation between segment and multisegment.
+
+    Time complexity:
+        ``O(segments_count)``
+    Memory complexity:
+        ``O(segments_count)``
+
+    where ``segments_count = len(multisegment)``.
+
+    :param segment: segment to check for.
+    :param multisegment: multisegment to check in.
+    :returns: relation between segment and multisegment.
+
+    >>> multisegment = [((0, 0), (1, 1)), ((1, 1), (3, 3))]
+    >>> segment_in_multisegment(((0, 0), (1, 0)),
+    ...                         multisegment) is Relation.TOUCH
+    True
+    >>> segment_in_multisegment(((0, 0), (0, 1)),
+    ...                         multisegment) is Relation.TOUCH
+    True
+    >>> segment_in_multisegment(((0, 1), (1, 0)),
+    ...                         multisegment) is Relation.CROSS
+    True
+    >>> segment_in_multisegment(((0, 0), (1, 1)),
+    ...                         multisegment) is Relation.COMPONENT
+    True
+    >>> segment_in_multisegment(((0, 0), (3, 3)),
+    ...                         multisegment) is Relation.EQUAL
+    True
+    >>> segment_in_multisegment(((2, 2), (4, 4)),
+    ...                         multisegment) is Relation.OVERLAP
+    True
+    >>> segment_in_multisegment(((4, 4), (5, 5)),
+    ...                         multisegment) is Relation.DISJOINT
+    True
+    """
+    return _multisegment.relate_segment(multisegment, segment)
+
+
+def multisegment_in_multisegment(left: Multisegment,
+                                 right: Multisegment) -> Relation:
+    """
+    Finds relation between multisegments.
+
+    Time complexity:
+        ``O(segments_count * log segments_count)``
+    Memory complexity:
+        ``O(segments_count)``
+
+    where ``segments_count = len(left) + len(right)``.
+
+    :param left: multisegment to check for.
+    :param right: multisegment to check in.
+    :returns: relation between multisegments.
+
+    >>> triangle_edges = [((0, 0), (1, 0)), ((1, 0), (0, 1)), ((0, 1), (0, 0))]
+    >>> square_edges = [((0, 0), (1, 0)), ((1, 0), (1, 1)), ((1, 1), (0, 1)),
+    ...                 ((0, 1), (0, 0))]
+    >>> multisegment_in_multisegment([], triangle_edges) is Relation.DISJOINT
+    True
+    >>> multisegment_in_multisegment(triangle_edges,
+    ...                              triangle_edges) is Relation.EQUAL
+    True
+    >>> multisegment_in_multisegment(triangle_edges,
+    ...                              square_edges) is Relation.OVERLAP
+    True
+    >>> multisegment_in_multisegment(square_edges,
+    ...                              triangle_edges) is Relation.OVERLAP
+    True
+    >>> multisegment_in_multisegment(square_edges,
+    ...                              square_edges) is Relation.EQUAL
+    True
+    """
+    return _multisegment.relate_multisegment(right, left)
+
+
 def point_in_contour(point: Point, contour: Contour) -> Relation:
     """
     Finds relation between point and contour.
@@ -134,6 +246,41 @@ def segment_in_contour(segment: Segment, contour: Contour) -> Relation:
     return _contour.relate_segment(contour, segment)
 
 
+def multisegment_in_contour(multisegment: Multisegment,
+                            contour: Contour) -> Relation:
+    """
+    Finds relation between multisegment and contour.
+
+    Time complexity:
+        ``O(segments_count * log segments_count)``
+    Memory complexity:
+        ``O(segments_count)``
+
+    where ``segments_count = len(left) + len(right)``.
+
+    :param multisegment: multisegment to check for.
+    :param contour: contour to check in.
+    :returns: relation between multisegment and contour.
+
+    >>> triangle_edges = [((0, 0), (1, 0)), ((1, 0), (0, 1)), ((0, 1), (0, 0))]
+    >>> square_edges = [((0, 0), (1, 0)), ((1, 0), (1, 1)), ((1, 1), (0, 1)),
+    ...                 ((0, 1), (0, 0))]
+    >>> triangle = [(0, 0), (1, 0), (0, 1)]
+    >>> square = [(0, 0), (1, 0), (1, 1), (0, 1)]
+    >>> multisegment_in_contour([], triangle) is Relation.DISJOINT
+    True
+    >>> multisegment_in_contour(triangle_edges, triangle) is Relation.EQUAL
+    True
+    >>> multisegment_in_contour(triangle_edges, square) is Relation.OVERLAP
+    True
+    >>> multisegment_in_contour(square_edges, triangle) is Relation.OVERLAP
+    True
+    >>> multisegment_in_contour(square_edges, square) is Relation.EQUAL
+    True
+    """
+    return _contour.relate_multisegment(contour, multisegment)
+
+
 def contour_in_contour(left: Contour, right: Contour) -> Relation:
     """
     Finds relation between contours.
@@ -227,6 +374,41 @@ def segment_in_region(segment: Segment, region: Region) -> Relation:
     return _region.relate_segment(region, segment)
 
 
+def multisegment_in_region(multisegment: Multisegment,
+                           region: Region) -> Relation:
+    """
+    Finds relation between multisegment and region.
+
+    Time complexity:
+        ``O(segments_count * log segments_count)``
+    Memory complexity:
+        ``O(segments_count)``
+
+    where ``segments_count = len(multisegment) + len(region).
+
+    :param multisegment: multisegment to check for.
+    :param region: region to check in.
+    :returns: relation between multisegment and region.
+
+    >>> triangle_edges = [((0, 0), (1, 0)), ((1, 0), (0, 1)), ((0, 1), (0, 0))]
+    >>> square_edges = [((0, 0), (1, 0)), ((1, 0), (1, 1)), ((1, 1), (0, 1)),
+    ...                 ((0, 1), (0, 0))]
+    >>> triangle = [(0, 0), (1, 0), (0, 1)]
+    >>> square = [(0, 0), (1, 0), (1, 1), (0, 1)]
+    >>> multisegment_in_region([], triangle) is Relation.DISJOINT
+    True
+    >>> multisegment_in_region(triangle_edges, triangle) is Relation.COMPONENT
+    True
+    >>> multisegment_in_region(triangle_edges, square) is Relation.ENCLOSED
+    True
+    >>> multisegment_in_region(square_edges, triangle) is Relation.TOUCH
+    True
+    >>> multisegment_in_region(square_edges, square) is Relation.COMPONENT
+    True
+    """
+    return _region.relate_multisegment(region, multisegment)
+
+
 def contour_in_region(contour: Contour, region: Region) -> Relation:
     """
     Finds relation between contour and region.
@@ -361,6 +543,46 @@ def segment_in_multiregion(segment: Segment,
     return _multiregion.relate_segment(multiregion, segment)
 
 
+def multisegment_in_multiregion(multisegment: Multisegment,
+                                multiregion: Multiregion) -> Relation:
+    """
+    Finds relation between multisegment and multiregion.
+
+    Time complexity:
+        ``O(segments_count * log segments_count)``
+    Memory complexity:
+        ``O(segments_count)``
+
+    where ``segments_count = len(multisegment) + sum(map(len, multiregion))``.
+
+    :param multisegment: multisegment to check for.
+    :param multiregion: multiregion to check in.
+    :returns: relation between multisegment and multiregion.
+
+    >>> triangle_edges = [((0, 0), (1, 0)), ((1, 0), (0, 1)), ((0, 1), (0, 0))]
+    >>> square_edges = [((0, 0), (1, 0)), ((1, 0), (1, 1)), ((1, 1), (0, 1)),
+    ...                 ((0, 1), (0, 0))]
+    >>> triangle = [(0, 0), (1, 0), (0, 1)]
+    >>> square = [(0, 0), (1, 0), (1, 1), (0, 1)]
+    >>> multisegment_in_multiregion([], []) is Relation.DISJOINT
+    True
+    >>> multisegment_in_multiregion([], [triangle]) is Relation.DISJOINT
+    True
+    >>> (multisegment_in_multiregion(triangle_edges, [triangle])
+    ...  is Relation.COMPONENT)
+    True
+    >>> (multisegment_in_multiregion(triangle_edges, [square])
+    ...  is Relation.ENCLOSED)
+    True
+    >>> multisegment_in_multiregion(square_edges, [triangle]) is Relation.TOUCH
+    True
+    >>> (multisegment_in_multiregion(square_edges, [square])
+    ...  is Relation.COMPONENT)
+    True
+    """
+    return _multiregion.relate_multisegment(multiregion, multisegment)
+
+
 def contour_in_multiregion(contour: Contour,
                            multiregion: Multiregion) -> Relation:
     """
@@ -397,7 +619,7 @@ def region_in_multiregion(region: Region,
     Finds relation between region and multiregion.
 
     Time complexity:
-        ``O(vertices_count * log (vertices_count))``
+        ``O(vertices_count * log vertices_count)``
     Memory complexity:
         ``O(vertices_count)``
 
@@ -536,18 +758,59 @@ def segment_in_polygon(segment: Segment, polygon: Polygon) -> Relation:
     return _polygon.relate_segment(polygon, segment)
 
 
+def multisegment_in_polygon(multisegment: Multisegment,
+                            polygon: Polygon) -> Relation:
+    """
+    Finds relation between multisegment and polygon.
+
+    Time complexity:
+        ``O(segments_count * log segments_count)``
+    Memory complexity:
+        ``O(segments_count)``
+
+    where ``segments_count = len(multisegment)\
+ + len(border) + sum(map(len, holes))``,
+    ``border, holes = polygon``.
+
+
+    :param multisegment: multisegment to check for.
+    :param polygon: polygon to check in.
+    :returns: relation between multisegment and polygon.
+
+    >>> triangle_edges = [((0, 0), (1, 0)), ((1, 0), (0, 1)), ((0, 1), (0, 0))]
+    >>> square_edges = [((0, 0), (1, 0)), ((1, 0), (1, 1)), ((1, 1), (0, 1)),
+    ...                 ((0, 1), (0, 0))]
+    >>> triangle = [(0, 0), (1, 0), (0, 1)]
+    >>> square = [(0, 0), (1, 0), (1, 1), (0, 1)]
+    >>> multisegment_in_polygon([], (triangle, [])) is Relation.DISJOINT
+    True
+    >>> (multisegment_in_polygon(triangle_edges, (triangle, []))
+    ...  is Relation.COMPONENT)
+    True
+    >>> (multisegment_in_polygon(triangle_edges, (square, []))
+    ...  is Relation.ENCLOSED)
+    True
+    >>> multisegment_in_polygon(square_edges, (triangle, [])) is Relation.TOUCH
+    True
+    >>> (multisegment_in_polygon(square_edges, (square, []))
+    ...  is Relation.COMPONENT)
+    True
+    """
+    return _polygon.relate_multisegment(polygon, multisegment)
+
+
 def contour_in_polygon(contour: Contour, polygon: Polygon) -> Relation:
     """
     Finds relation between contour and polygon.
 
     Time complexity:
-        ``O(vertices_count * log (vertices_count))``
+        ``O(vertices_count * log vertices_count)``
     Memory complexity:
         ``O(vertices_count)``
 
     where ``vertices_count = len(contour)\
  + len(border) + sum(map(len, holes))``,
-    ``border, holes = polygon``
+    ``border, holes = polygon``.
 
     :param contour: contour to check for.
     :param polygon: polygon to check in.
@@ -572,13 +835,13 @@ def region_in_polygon(region: Region, polygon: Polygon) -> Relation:
     Finds relation between region and polygon.
 
     Time complexity:
-        ``O(vertices_count * log (vertices_count))``
+        ``O(vertices_count * log vertices_count)``
     Memory complexity:
         ``O(vertices_count)``
 
     where ``vertices_count = len(region)\
  + len(border) + sum(map(len, holes))``,
-    ``border, holes = polygon``
+    ``border, holes = polygon``.
 
     :param region: region to check for.
     :param polygon: polygon to check in.
@@ -604,13 +867,13 @@ def multiregion_in_polygon(multiregion: Multiregion,
     Finds relation between multiregion and polygon.
 
     Time complexity:
-        ``O(vertices_count * log (vertices_count))``
+        ``O(vertices_count * log vertices_count)``
     Memory complexity:
         ``O(vertices_count)``
 
     where ``vertices_count = sum(map(len, multiregion))\
  + len(border) + sum(map(len, holes))``,
-    ``border, holes = polygon``
+    ``border, holes = polygon``.
 
     :param multiregion: multiregion to check for.
     :param polygon: polygon to check in.
@@ -635,7 +898,7 @@ def polygon_in_polygon(left: Polygon, right: Polygon) -> Relation:
     Checks if the polygon fully lies inside the other one.
 
     Time complexity:
-        ``O(vertices_count * log (vertices_count))``
+        ``O(vertices_count * log vertices_count)``
     Memory complexity:
         ``O(vertices_count)``
 
